/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "CvfdStructuringElement.h"

using namespace std;
using namespace FD;

class CvfdMorphology;
   
DECLARE_NODE(CvfdMorphology)
/*Node
 *
 * @name CvfdMorphology
 * @category FDOPENCV_ORG:Morphological Operations
 * @description Performs advanced morphological transformations
 *
 * @input_name IMAGEIN
 * @input_description Source Image
 * @input_type CvfdImage
 *
 * @input_name ELEMENT
 * @input_description Structuring element
 * @input_type CvfdStructuringElement
 *
 * @parameter_name OPERATION
 * @parameter_type string
 * @parameter_value CV_MOP_OPEN;CV_MOP_CLOSE;CV_MOP_GRADIENT;CV_MOP_TOPHAT;CV_MOP_BLACKHAT
 * @parameter_description Type of morphological operation
 *
 * @parameter_name ITERATIONS
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Number of times erosion and dilation are applied
 *
 * @output_name IMAGEOUT
 * @output_description Destination image
 * @output_type CvfdImage
 *
 END*/
   
class CvfdMorphology : public BufferedNode {
      
  //Input ID
  int m_imageInID;
  int m_elementID;
  //Output ID
  int m_imageOutID;
  //Parameters
  int m_iterations;
  string m_operation;
  //map of the line_type
  map<string,int> m_operationMap;
      
public:
  CvfdMorphology(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_imageInID = addInput("IMAGEIN");
    m_elementID = addInput("ELEMENT");
         
    //add outputs
    m_imageOutID = addOutput("IMAGEOUT");
         
    //Initialize parameters         
    m_iterations = dereference_cast<int>(parameters.get("ITERATIONS"));
    m_operation = object_cast<String>(parameters.get("OPERATION"));         
         
    //Initialize the lineType map
    m_operationMap["CV_MOP_OPEN"] = CV_MOP_OPEN;
    m_operationMap["CV_MOP_CLOSE"] = CV_MOP_CLOSE;
    m_operationMap["CV_MOP_GRADIENT"] = CV_MOP_GRADIENT;   
    m_operationMap["CV_MOP_TOPHAT"] = CV_MOP_TOPHAT;
    m_operationMap["CV_MOP_BLACKHAT"] = CV_MOP_BLACKHAT;          
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {           
    //Read the inputs
    RCPtr<CvfdStructuringElement> elementPtr = getInput(m_elementID,count);
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);
         
    //Handle
         
    //CvfdImage* imageTemp = imagePtr->zero();
    //CvfdImage* imageOut = imagePtr->zero();
    CvfdImage* imageTemp= new CvfdImage(imagePtr->getImage());  
    CvfdImage* imageOut= new CvfdImage(imagePtr->getImage()); 
         
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    __BEGIN__; 
    OPENCV_CALL(cvMorphologyEx(imagePtr->getImage(), imageOut->getImage(), imageTemp->getImage(), elementPtr->getStructuringElement(), m_operationMap[m_operation], m_iterations)); 
    __END__;
    cvSetErrMode( status );
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to morphology the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      } 
    delete imageTemp;         
    out[count] = ObjectRef(imageOut);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdMorphology)
};
